package org.t2framework.lucy;

import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.lucy.exception.TooManyRegistrationException;

/**
 * {@.en Strategy class for getting BeanDesc or no BeanDesc.}
 * 
 * <br />
 * 
 * {@.ja BeanDescを取得または無いときの振る舞いを定義したクラスです.}
 * 
 * @author shot
 */
public interface BeanDescStrategy {

	/**
	 * {@.en Get {@link BeanDesc} from the {@link Class} and {@link List
	 * <BeanDesc<T>>} . By default, return {@link BeanDesc} if there is one
	 * component in {@link List}.If more than one, throw
	 * {@link TooManyRegistrationException}.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescを与えられたClassとBeanDescのListから取得します．
	 * デフォルトの挙動では、List内に一つしかBeanDescが無い場合、それを返します．複数ある場合、{@code
	 * TooManyRegistrationException}が発生します．}
	 * 
	 * @param <T>
	 * @param beanClass
	 * @param beanDescList
	 * @throws TooManyRegistrationException
	 * @return
	 * @see org.t2framework.lucy.impl.DefaultBeanDescStrategyImpl
	 * @see org.t2framework.lucy.exception.TooManyRegistrationException
	 */
	<T> BeanDesc<T> getBeanDesc(Class<T> beanClass,
			List<BeanDesc<T>> beanDescList) throws TooManyRegistrationException;

	/**
	 * {@.en Handle if there is no {@link BeanDesc}.By default, just return
	 * null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescが存在しない場合の挙動を決めるメソッドです．デフォルトでは、nullを返します．}
	 * 
	 * @param <T>
	 * @param key
	 * @return
	 * @see org.t2framework.lucy.impl.DefaultBeanDescStrategyImpl
	 */
	<T> T handleNoBeanDesc(Object key);
}
