package org.t2framework.lucy.annotation.core;

import static java.lang.annotation.ElementType.METHOD;
import static java.lang.annotation.ElementType.CONSTRUCTOR;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.Retention;
import java.lang.annotation.Target;

/**
 * 
 * {@.en &#064;Inject annotation is the annotation to inject component.<br />
 * 
 * Lucy supports method injection, setter injection, and constructor injection. <br />
 * (currently in now, but nobody knows in the future) Lucy recommends to use
 * method injection that method name starts with inject, such as {@code
 * inject(Server server, Configuration config)}. }
 * 
 * <br />
 * 
 * {@.ja &#064;Injectアノテーションはコンポーネントへのインジェクトポイントを明記するためのアノテーションです
 * 
 * {@code Lucy}はメソッドインジェクション、セッターインジェクション、コンストラクタインジェクションをサポートしています．
 * その中でも複数のコンポーネントをまとめてインジェクトできる、シングルポイントにインジェクトされる点がまとまっているなどの理由から、
 * メソッドインジェクションを推奨しています．例えば以下のようになります:
 * 
 * <pre>
 * 
 * &#064;Inject public void inject(Server server, Configuration config) { .... }
 * 
 * </pre>
 * 
 * @author shot
 * 
 */
@Retention(RUNTIME)
@Target( { METHOD, CONSTRUCTOR })
public @interface Inject {

	String TYPES = "types";

	String DEFAUL_NAME = "";

	int DEFAULT_NAME_LENGTH = 1;

	/**
	 * 
	 * {@.en types() are used when inject multiple component to one method.}
	 * 
	 * <br />
	 * 
	 * {@.ja 複数コンポーネントインジェクションするときに、型補完するためのメソッドです．}
	 * 
	 * @return
	 */
	Class<?>[] types() default Object.class;

	/**
	 * 
	 * {@.en Inject by name.}
	 * 
	 * <br />
	 * 
	 * {@.ja 名前によるインジェクションです．}
	 * 
	 * @return
	 */
	String[] name() default "";
}
