package org.t2framework.lucy.aop.impl;

import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.AopEngine;
import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Logger;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.EnhancedClassBuilder;
import org.t2framework.lucy.aop.EnhancedClassCache;
import org.t2framework.lucy.aop.InterceptorDescStrategy;

/**
 * 
 * @author shot
 * @author c9katayama
 */
public abstract class AbstractAopEngine implements AopEngine,
		EnhancedClassCache {

	protected static Logger logger = Logger.getLogger(AbstractAopEngine.class);

	protected InterceptorDescStrategy strategy;

	protected Map<Class<?>, Class<?>> classCache = CollectionsUtil
			.newSynchronizedWeakHashMap();

	public AbstractAopEngine() {
		strategy = new InterceptorDescStrategyImpl();
	}

	public AbstractAopEngine(InterceptorDescStrategy strategy) {
		this.strategy = Assertion.notNull(strategy);
	}

	@Override
	public void startup() {
	}

	@Override
	public <T> Class<? extends T> getEnhancedClass(
			Class<? extends T> originalClass, List<Aspect> aspects) {
		Class<? extends T> enhancedClass = get(originalClass);
		if (enhancedClass == null) {
			enhancedClass = buildEnhancedClass(originalClass, aspects);
			cache(originalClass, enhancedClass);
		}
		return enhancedClass;
	}

	protected synchronized <T> Class<T> buildEnhancedClass(
			Class<? extends T> originalClass, List<Aspect> aspects) {
		AspectDesc aspectDesc = createAspectDesc(originalClass, aspects);
		final EnhancedClassBuilder<T> builder = createBuilder(strategy);
		Class<T> enhancedClass = builder.build(aspectDesc);
		return enhancedClass;
	}

	protected abstract <T> AspectDesc createAspectDesc(Class<T> originalClass,
			List<Aspect> aspects);

	protected abstract <T> EnhancedClassBuilder<T> createBuilder(
			InterceptorDescStrategy strategy);

	@Override
	public void shutdown() {
		classCache.clear();
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> void cache(final Class<? extends T> originalClass,
			final Class<? extends T> enhancedClass) {
		Assertion.notNulls(originalClass, enhancedClass);
		classCache.put(originalClass, enhancedClass);
	}

	@Override
	public <T> void clear(Class<? extends T> originalClass) {
		classCache.remove(originalClass);
	}

	@Override
	public void clear() {
		classCache.clear();
	}

	@SuppressWarnings("unchecked")
	@Override
	public <T> Class<? extends T> get(Class<? extends T> originalClass) {
		return (Class<? extends T>) classCache.get(originalClass);
	}
}
