package org.t2framework.lucy.aop.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.aop.AspectDesc;

public class AspectDescImpl implements AspectDesc {

	protected Class<?> originalClass;

	protected Class<?> enhancedClass;

	protected String enhancedClassName;

	protected List<Aspect> aspects = Collections.emptyList();

	protected Map<String, Object> parameters = null;

	protected boolean interface_ = false;

	public AspectDescImpl(Class<?> originalClass, List<Aspect> aspects) {
		this(originalClass, null, aspects, new HashMap<String, Object>());
	}

	public AspectDescImpl(Class<?> originalClass, String enhancedClassName,
			List<Aspect> aspects) {
		this(originalClass, enhancedClassName, aspects,
				new HashMap<String, Object>());
	}

	public AspectDescImpl(Class<?> originalClass, List<Aspect> aspects,
			Map<String, Object> parameters) {
		this(originalClass, null, aspects, parameters);
	}

	public AspectDescImpl(Class<?> originalClass, String enhancedClassName,
			List<Aspect> aspects, Map<String, Object> parameters) {
		this.originalClass = Assertion.notNull(originalClass);
		this.enhancedClassName = enhancedClassName;
		this.aspects = Assertion.notNull(aspects);
		this.parameters = Assertion.notNull(parameters);
		this.interface_ = this.originalClass.isInterface();
	}

	@Override
	public List<Aspect> getAspects() {
		return aspects;
	}

	@Override
	public Class<?> getEnhancedClass() {
		return enhancedClass;
	}

	@Override
	public String getEnhancedClassName() {
		return enhancedClassName;
	}

	@Override
	public Class<?> getOriginalClass() {
		return originalClass;
	}

	@Override
	public Map<String, Object> getParameters() {
		return parameters;
	}

	@Override
	public void setEnhancedClass(Class<?> enhancedClass) {
		this.enhancedClass = enhancedClass;
	}

	@Override
	public void setEnhancedClassName(String name) {
		this.enhancedClassName = name;
	}

	@Override
	public boolean isInterfaceTarget() {
		return interface_;
	}

}