package org.t2framework.lucy.aop.javassist.meta.impl;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Map;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.CtMethod;
import javassist.NotFoundException;

import org.t2framework.commons.Constants;
import org.t2framework.commons.meta.spi.AnnotationConfigSupport;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Reflections.ClassLoaderUtil;
import org.t2framework.commons.util.Reflections.ClassUtil;
import org.t2framework.commons.util.Reflections.FieldUtil;
import org.t2framework.commons.util.Reflections.MethodUtil;
import org.t2framework.lucy.aop.exception.CannotCompileRuntimeException;
import org.t2framework.lucy.aop.javassist.ClassEnhanceHelper.LucyCtNewMethod;
import org.t2framework.lucy.aop.javassist.util.ClassPoolUtil;
import org.t2framework.lucy.aop.javassist.util.JavassistUtil;
import org.t2framework.lucy.aop.javassist.util.LucyClassPool;

/**
 * 
 * {@.en AnnotationDescSupportImpl is the class to create annotation with
 * changed value using AOP.See test class if you would like to know what we
 * would like to do.
 * 
 * This class is very experimental class, so not stable. }
 * 
 * <br />
 * 
 * {@.ja Annotationを書き換えるクラスです.}
 * 
 * @author shot
 * 
 */
public class AnnotationConfigSupportImpl implements AnnotationConfigSupport {

	public static final String REPLACED_OBJECT_PREFIX = "$$REPLACED_OBJECT_";

	@SuppressWarnings("unchecked")
	@Override
	public synchronized <T extends Annotation> T createAnnotation(
			final T original, final String targetMethodName,
			final Object replaceValue) {
		Assertion.notNull(original, "original");
		Assertion.notNull(targetMethodName, "targetMethodName");
		Assertion.notNull(replaceValue, "value");

		final Class<? extends Annotation> orgClass = original.annotationType();
		final LucyClassPool pool = ClassPoolUtil.getClassPool(orgClass);
		final CtClass annotationCtClass = createNewCtClass(orgClass, pool);

		addAnnotationTypeMethod(orgClass, annotationCtClass, pool);

		final Map<String, Object> replacedValueMap = CollectionsUtil
				.newHashMap();
		for (Method m : orgClass.getDeclaredMethods()) {
			final String methodName = m.getName();
			final boolean valueReplace = methodName.equals(targetMethodName);
			Class<?> returnType = m.getReturnType();
			final Object value = (valueReplace) ? replaceValue : MethodUtil
					.invoke(m, original, Constants.EMPTY_ARRAY);
			StringBuilder body = new StringBuilder(50);
			if (returnType == String.class) {
				AnnotationWriterUtil.writeStringBody(value, body);
			} else if (returnType == Class.class) {
				AnnotationWriterUtil.writeClassBody(value, body);
			} else if (Enum.class.isAssignableFrom(returnType)) {
				AnnotationWriterUtil.writeEnumBody(value, body);
			} else if (Annotation.class.isAssignableFrom(returnType)) {
				final String fieldName = getFieldName(annotationCtClass, value);
				createAdditionalField(fieldName, ((Annotation) value)
						.annotationType(), annotationCtClass);
				AnnotationWriterUtil.writeObjectBody(fieldName, body);
				replacedValueMap.put(fieldName, value);
			} else if (returnType.isArray()) {
				final String fieldName = getFieldName(annotationCtClass, value);
				createAdditionalField(fieldName, value, annotationCtClass);
				AnnotationWriterUtil.writeObjectBody(fieldName, body);
				replacedValueMap.put(fieldName, value);
			} else {
				AnnotationWriterUtil.writeObjectBody(value, body);
			}
			createAndAddCtMethod(annotationCtClass, pool, m, methodName,
					new String(body));
		}
		final ClassLoader classLoader = ClassLoaderUtil.getClassLoader(original
				.annotationType());
		try {
			Class<?> retClass = JavassistUtil.toClass(annotationCtClass,
					classLoader);
			if (replacedValueMap.isEmpty() == false) {
				setupArrayFields(retClass, replacedValueMap);
			}
			return (T) ClassUtil.newInstance(retClass);
		} finally {
			replacedValueMap.clear();
			destroy(annotationCtClass);
		}
	}

	protected void setupArrayFields(Class<?> retClass,
			Map<String, Object> arrayValueMap) {
		for (Map.Entry<String, Object> entry : arrayValueMap.entrySet()) {
			final String key = entry.getKey();
			final Object value = entry.getValue();
			final Field field = FieldUtil.getField(retClass, key);
			FieldUtil.set(field, null, value);
		}
	}

	protected void createAdditionalField(String fieldName, Object value,
			CtClass declaringCtClass) {
		createAdditionalField(fieldName, value.getClass(), declaringCtClass);
	}

	protected void createAdditionalField(String fieldName, Class<?> type,
			CtClass declaringCtClass) {
		LucyClassPool pool = ClassPoolUtil.getClassPool(type);
		CtClass ctClass = JavassistUtil.getCtClass(pool, type);
		try {
			CtField field = new CtField(ctClass, fieldName, declaringCtClass);
			JavassistUtil.setCtFieldAsPublicStatic(field);
			declaringCtClass.addField(field);
		} catch (CannotCompileException e) {
			throw new CannotCompileRuntimeException(e);
		}
	}

	protected String getFieldName(CtClass annotationCtClass, Object value) {
		int i = 0;
		String name = null;
		while (true) {
			name = REPLACED_OBJECT_PREFIX + Integer.toString(i);
			try {
				annotationCtClass.getDeclaredField(name);
			} catch (NotFoundException e) {
				break;
			}
			i++;
		}
		return name;
	}

	protected void destroy(CtClass ctClass) {
		ctClass.detach();
		ctClass = null;
	}

	protected CtClass createNewCtClass(
			Class<? extends Annotation> annotationClass, ClassPool pool) {
		final String name = annotationClass.getName();
		final String classNameCandidate = name + "$$" + System.nanoTime();
		final CtClass replacedCtClass = JavassistUtil.makeCtClass(pool,
				classNameCandidate, Object.class);
		replacedCtClass.setInterfaces(new CtClass[] { JavassistUtil.getCtClass(
				pool, annotationClass) });
		JavassistUtil.addDefaultConstructor(replacedCtClass);
		return replacedCtClass;
	}

	protected void addAnnotationTypeMethod(
			Class<? extends Annotation> annotationClass, CtClass ctClass,
			LucyClassPool pool) {
		Method annotationTypeMethod = MethodUtil.getMethod(annotationClass,
				ANNOTATIONTYPE_METHOD_NAME, Constants.EMPTY_CLASS_ARRAY);
		String annotationTypeBody = "{return " + annotationClass.getName()
				+ ".class;}";
		createAndAddCtMethod(ctClass, pool, annotationTypeMethod,
				ANNOTATIONTYPE_METHOD_NAME, annotationTypeBody);
	}

	protected void createAndAddCtMethod(CtClass ctClass, LucyClassPool pool,
			Method method, String methodName, String body) {
		CtMethod annotationTypeCtMethod = LucyCtNewMethod.make(ctClass, pool,
				method, methodName, body);
		JavassistUtil.addCtMethod(ctClass, annotationTypeCtMethod);
	}
}
