package org.t2framework.lucy.aop.module;

import org.t2framework.commons.aop.AopEngine;
import org.t2framework.commons.meta.spi.AnnotationConfigSupport;
import org.t2framework.commons.util.Reflections.ClassUtil;
import org.t2framework.lucy.aop.util.AopUtil;
import org.t2framework.lucy.config.stax.XmlEventHandler;
import org.t2framework.lucy.module.LucyAopModule;
import org.t2framework.lucy.spi.Behavior;

/**
 * Select proxy-based AOP module or Javassist-based AOP module.
 * 
 * @author c9katayama
 * 
 */
public class SelectableLucyAopModuleImpl implements LucyAopModule {

	protected LucyAopModule selectModule;

	public SelectableLucyAopModuleImpl() {
		selectModule();
	}

	protected void selectModule() {
		if (AopUtil.isJavassistEnabled()) {
			selectModule = ClassUtil
					.newInstanceByName("org.t2framework.lucy.aop.javassist.module.JavassistAopModuleImpl");
		} else {
			selectModule = ClassUtil
					.newInstanceByName("org.t2framework.lucy.aop.proxy.module.ProxyAopModuleImpl");
		}
	}

	@Override
	public XmlEventHandler getAspectTagHandler() {
		return selectModule.getAspectTagHandler();
	}

	@Override
	public <S> S getService(Class<? super S> serviceClass) {
		return (S) selectModule.getService(serviceClass);
	}

	@Override
	public Class<? extends Behavior> getAspectBehaviorClass() {
		return selectModule.getAspectBehaviorClass();
	}

	@Override
	public AnnotationConfigSupport getAnnotationConfigSupport() {
		return selectModule.getAnnotationConfigSupport();
	}

	@Override
	public AopEngine getAopEngine() {
		return selectModule.getAopEngine();
	}
}
