package org.t2framework.lucy.aop.proxy;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Aspect;
import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.commons.util.Reflections.MethodUtil;
import org.t2framework.lucy.aop.AspectDesc;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.InterceptorDescStrategy;
import org.t2framework.lucy.aop.impl.InterceptorDescStrategyImpl;

/**
 * @author c9katayama
 */
public class ProxyInterceptorDescStrategyImpl extends
		InterceptorDescStrategyImpl implements InterceptorDescStrategy {

	@Override
	protected <T> void assemble0(Map<AccessibleObject, InterceptorDesc> map,
			MethodDesc methodDesc, AspectDesc aspectDesc) {
		final List<Interceptor> list = CollectionsUtil.newArrayList();
		if (methodDesc == null) {
			return;
		}
		for (Aspect a : aspectDesc.getAspects()) {
			LOG.log("DLucyAop0005", methodDesc.getDeclaringClass().getName(),
					methodDesc.getMethodName(), a.getPointcut()
							.getPatternString());
			if (a.getPointcut().isApplied(methodDesc.getMethod())) {
				list.addAll(a.getInterceptors());
				LOG.log("DLucyAop0006", methodDesc.getDeclaringClass()
						.getName(), methodDesc.getMethodName());
			} else {
				LOG.log("DLucyAop0007", methodDesc.getDeclaringClass()
						.getName(), methodDesc.getMethodName());
			}
		}
		if (list.size() == 0) {
			return;
		}
		Method targetMethod = methodDesc.getMethod();
		Class<?>[] interfaces = aspectDesc.getEnhancedClass().getInterfaces();
		for (int i = 0; i < interfaces.length; i++) {
			Method interfaceMethod = MethodUtil.getMethodNoException(
					interfaces[i], targetMethod.getName(), targetMethod
							.getParameterTypes());
			if (interfaceMethod != null) {
				map.put(interfaceMethod,
						createInterceptorDesc(methodDesc, list));
			}
		}
	}
}
