package org.t2framework.lucy.aop.proxy;

import java.lang.reflect.AccessibleObject;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.util.List;
import java.util.Map;

import org.t2framework.commons.aop.spi.Interceptor;
import org.t2framework.commons.aop.spi.Invocation;
import org.t2framework.lucy.aop.InterceptorDesc;
import org.t2framework.lucy.aop.impl.InvocationImpl;

/**
 * Proxy AOP invocation handler.
 * 
 * @author c9katayama
 */
public class ProxyInvocationHandler implements InvocationHandler {

	protected Map<AccessibleObject, InterceptorDesc> proxyInterceptorMap;

	protected Object thisInstance;

	public ProxyInvocationHandler(
			Map<AccessibleObject, InterceptorDesc> proxyInterceptorMap,
			Object thisInstance) {
		this.proxyInterceptorMap = proxyInterceptorMap;
		this.thisInstance = thisInstance;
	}

	@Override
	public Object invoke(Object proxyInstance, Method method, Object[] args)
			throws Throwable {

		InterceptorDesc interceptorDesc = proxyInterceptorMap.get(method);
		if (interceptorDesc != null) {
			List<Interceptor> interceptorList = interceptorDesc
					.getInterceptors();
			if (interceptorList != null && interceptorList.size() > 0) {
				Invocation<? extends AccessibleObject> invocation = new InvocationImpl(
						interceptorList, method, method, args, proxyInstance
								.getClass(), thisInstance);
				Object result = invocation.proceed();
				return result;
			}
		}
		Object result = method.invoke(thisInstance, args);
		return result;
	}
}
