package org.t2framework.lucy.behavior;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.inject.ConstructorInjector;
import org.t2framework.lucy.inject.Injector;
import org.t2framework.lucy.inject.SimpleConstructorInjector;
import org.t2framework.lucy.spi.AbstractBehavior;
import org.t2framework.lucy.spi.Behavior;

/**
 * 
 * {@.en {@link AbstractComponentCreationBehavior} is a behavior for creating
 * component. }
 * 
 * <br />
 * 
 * {@.ja コンポーネントの作成を実施するタイプの{@link Behavior}の共通抽象クラスです．}
 * 
 * @author shot
 * 
 */
public abstract class AbstractComponentCreationBehavior extends
		AbstractBehavior {

	protected Lock lock = new ReentrantLock();

	protected final Injector injector;

	public AbstractComponentCreationBehavior(BeanDesc<?> beanDesc,
			ClassDesc<?> classDesc, Config cd) {
		super(beanDesc, classDesc, cd);
		if (beanDesc.getConstructorDesc().hasOnlyDefaultConstructor()) {
			injector = new SimpleConstructorInjector(beanDesc);
		} else {
			injector = new ConstructorInjector(beanDesc);
		}
	}

	/**
	 * Execute component creation.
	 */
	@Override
	public <T> T execute(T t, Lucy lucy) {
		lock.lock();
		try {
			t = handleComponentCreation(t, lucy, getBeanDesc());
		} finally {
			lock.unlock();
		}
		return t;
	}

	protected abstract <T> T handleComponentCreation(T t, Lucy lucy,
			BeanDesc<Object> beanDesc);

	@Override
	public final Lifecycle getLifecycle() {
		return Lifecycle.COMPONENT_CREATING;
	}

	protected Injector getInjector() {
		return injector;
	}
}
