package org.t2framework.lucy.behavior;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import org.t2framework.commons.annotation.ConfigurationTarget;
import org.t2framework.commons.annotation.core.Singleton;
import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.config.meta.impl.SingletonInstanceConfig;

/**
 * {@link SingletonBehavior} is a singleton behavior for component. It creates
 * component at once, then caches into the {@link BeanDesc}.
 * 
 * @author shot
 */
@ConfigurationTarget( { Singleton.class, SingletonInstanceConfig.class })
public class SingletonBehavior extends AbstractComponentCreationBehavior {

	protected Lock lock = new ReentrantLock();

	public SingletonBehavior(BeanDesc<?> beanDesc, ClassDesc<?> classDesc,
			Config cd) {
		super(beanDesc, classDesc, cd);
	}

	@SuppressWarnings("unchecked")
	@Override
	protected <T> T handleComponentCreation(T t, Lucy lucy,
			BeanDesc<Object> beanDesc) {
		Object ret = beanDesc.getComponent();
		if (ret == null) {
			beanDesc.setSingleton(true);
			ret = getInjector().inject(t, lucy);
			beanDesc.setComponent(ret);
		}
		return (T) ret;
	}

}
