package org.t2framework.lucy.config.meta;

import java.util.LinkedList;
import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.ConstructorDesc;
import org.t2framework.commons.meta.ExpressionAware;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.meta.impl.BeanDescImpl;
import org.t2framework.commons.util.Assertion;

/**
 * 
 * {@.en }
 * 
 * <br />
 * 
 * {@.ja 設定ファイル経由で生成される特殊な{@link BeanDesc}です．{@link BeanDesc}生成後に
 * {@link MethodDesc}や{@link PropertyDesc}の置き換えなどを行います． }
 * 
 * @author shot
 * 
 * @param <T>
 */
public class ConfigurableBeanDesc<T> extends BeanDescImpl<T> {

	protected LinkedList<ExpressionAware> latests;

	protected String autoinject;

	public ConfigurableBeanDesc(Class<? extends T> componentClass) {
		super(componentClass);
	}

	public ConfigurableBeanDesc(ClassDesc<T> classDesc) {
		super(classDesc);
	}

	@Override
	protected void init0() {
		super.init0();
		this.latests = new LinkedList<ExpressionAware>();
	}

	@SuppressWarnings("unchecked")
	public <D extends ExpressionAware> D getLatest() {
		return (D) latests.pollFirst();
	}

	@Override
	public void addMethodDesc(MethodDesc md) {
		final String methodName = md.getMethodName();
		List<MethodDesc> list = methodDescMap.get(methodName);
		if (list == null) {
			super.addMethodDesc(md);
			return;
		}
		if (!replaceMethodDesc(list, md)) {
			super.addMethodDesc(md);
		}
		replaceMethodDesc(methodDescList, md);
		latests.offerFirst(md);
	}

	protected boolean replaceMethodDesc(List<MethodDesc> list, MethodDesc desc) {
		for (int i = 0; i < list.size(); ++i) {
			MethodDesc md = list.get(i);
			if (md.equals(desc)) {
				list.remove(i);
				list.add(i, desc);
				return true;
			}
		}
		return false;
	}

	@Override
	public void addPropertyDesc(PropertyDesc<T> pd) {
		Assertion.notNull(pd);
		final String propertyName = pd.getPropertyName();
		PropertyDesc<T> propertyDesc = propertyDescMap.get(propertyName);
		if (propertyDesc == null) {
			super.addPropertyDesc(pd);
			return;
		}
		propertyDescMap.put(propertyName, pd);
		replacePropertyDesc(propertyDescList, pd);
		latests.offerFirst(pd);
	}

	protected boolean replacePropertyDesc(List<PropertyDesc<T>> list,
			PropertyDesc<T> desc) {
		for (int i = 0; i < list.size(); ++i) {
			PropertyDesc<T> pd = list.get(i);
			if (pd.getPropertyName().equals(desc.getPropertyName())) {
				list.remove(i);
				list.add(i, desc);
				return true;
			}
		}
		return false;
	}

	@Override
	public void setConstructorDesc(ConstructorDesc<T> constructorDesc) {
		super.setConstructorDesc(constructorDesc);
		latests.offerFirst(constructorDesc);
	}

	public String getAutoinject() {
		return autoinject;
	}

	public void setAutoinject(String autoinject) {
		this.autoinject = autoinject;
	}

}
