package org.t2framework.lucy.el.mvel;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.mvel2.ParserContext;
import org.t2framework.commons.Disposable;
import org.t2framework.commons.Disposer;
import org.t2framework.commons.el.Expression;
import org.t2framework.commons.exception.IORuntimeException;
import org.t2framework.commons.module.Modules;
import org.t2framework.commons.util.ResourceUtil;
import org.t2framework.commons.util.StringUtil;
import org.t2framework.commons.util.Reflections.ClassUtil;
import org.t2framework.commons.util.Reflections.PackageUtil;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyBootstrap;
import org.t2framework.lucy.el.AbstractExpressionFactory;
import org.t2framework.lucy.el.ExpressionNameResolver;

/**
 * Mvel expression factory.
 * 
 * @author shot
 * 
 */
public class MVELExpressionFactory extends AbstractExpressionFactory {

	private static final String PATH = "META-INF/services/el.properties";

	protected Set<String> defaultPackages = new HashSet<String>();

	protected ExpressionNameResolver resolver;

	public MVELExpressionFactory() {
		Disposer.add(new Disposable() {
			@Override
			public void dispose() {
				clear();
			}
		});
		initialize();
	}

	protected void initialize() {
		defaultPackages.add(PackageUtil.getPackageName(Disposer.class));
		defaultPackages.add(PackageUtil.getPackageName(Modules.class));
		defaultPackages.add(PackageUtil.getPackageName(LucyBootstrap.class));
		defaultPackages.add(PackageUtil.getPackageName(StringUtil.class));
	}

	@Override
	public Map<String, Class<?>> resolve() {
		Map<String, Class<?>> ret = new HashMap<String, Class<?>>();
		InputStream is = ResourceUtil.getResourceAsStreamNoException(PATH);
		if (is == null) {
			return ret;
		}
		Properties props = new Properties();
		try {
			props.load(is);
		} catch (IOException e) {
			throw new IORuntimeException(e);
		}
		for (Enumeration<?> e = props.propertyNames(); e.hasMoreElements();) {
			String key = (String) e.nextElement();
			String value = props.getProperty(key);
			if (StringUtil.isEmpty(value)) {
				continue;
			}
			Class<?> c = ClassUtil.forNameNoException(value);
			if (c != null) {
				ret.put(key, c);
			}
		}
		final ExpressionNameResolver resolver = getExpressionNameResolver();
		if (resolver != null) {
			ret.putAll(resolver.resolve());
		}
		return ret;
	}

	/**
	 * Create {@link MVELExpression}.
	 */
	@Override
	protected Expression<Lucy> doCreateExpression(String source,
			Map<String, Class<?>> namingContext) {
		final ParserContext context = new ParserContext();
		for (String packageName : defaultPackages) {
			context.addPackageImport(packageName);
		}
		if (namingContext != null && namingContext.isEmpty() == false) {
			for (Map.Entry<String, Class<?>> entry : namingContext.entrySet()) {
				context.addImport(entry.getKey(), entry.getValue());
			}
		}
		return new MVELExpression(source, context);
	}

	@Override
	public void setExpressionNameResolver(ExpressionNameResolver resolver) {
		this.resolver = resolver;
	}

	public ExpressionNameResolver getExpressionNameResolver() {
		return resolver;
	}

	protected void clear() {
		this.resolver = null;
	}
}
