package org.t2framework.lucy.impl;

import java.util.List;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.commons.util.CollectionsUtil;
import org.t2framework.lucy.BeanDescStrategy;

/**
 * {@.en BeanDescBag is an wrapper class for BeanDesc to store.It is used for
 * component saving.}
 * 
 * <br />
 * 
 * {@.ja BeanDescBagは複数のBeanDescを保持するためのクラスです．
 * 単一のBeanDescを返す場合、どのような方針で返すかは{@code BeanDescStrategy}に依ります． }
 * 
 * @author Skirnir
 * @author shot
 */
public class BeanDescBag<T> {

	protected Class<T> beanClass;

	protected List<BeanDesc<T>> beanDescList = CollectionsUtil.newArrayList();

	protected BeanDescStrategy beanDescStrategy;

	/**
	 * {@.en Create {@code BeanDescBag} with bean class and {@code
	 * BeanDescStrategy}.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescBagを作成します． }
	 * 
	 * @param beanClass
	 * @param beanDescStrategy
	 */
	public BeanDescBag(Class<T> beanClass, BeanDescStrategy beanDescStrategy) {
		this.beanClass = beanClass;
		this.beanDescStrategy = beanDescStrategy;
	}

	/**
	 * {@.en Add {@code BeanDesc}. Not allowed null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescを足します．nullは許可されていません．}
	 * 
	 * @param beanDesc
	 */
	public void add(BeanDesc<T> beanDesc) {
		beanDescList.add(Assertion.notNull(beanDesc));
	}

	/**
	 * {@.en Get a {@code BeanDesc}. How to choose a BeanDesc simply depends on
	 * {@code BeanDescStrategy}.}
	 * 
	 * <br />
	 * 
	 * {@.ja
	 * BeanDescを返します．どのように単一のBeanDescを選択するかはBeanDescStrategyに依存しています．}
	 * 
	 * @return
	 */
	public BeanDesc<T> getBeanDesc() {
		return beanDescStrategy.getBeanDesc(beanClass, beanDescList);
	}

	/**
	 * {@.en Get list of {@code BeanDesc}. This method never returns null.}
	 * 
	 * <br />
	 * 
	 * {@.ja BeanDescのリストを返します．このメソッドはnullは返しません．}
	 * 
	 * @return
	 */
	public List<BeanDesc<T>> getBeanDescList() {
		return beanDescList;
	}
}
