package org.t2framework.lucy.inject;

import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.ConfigContainer;
import org.t2framework.lucy.annotation.core.Inject;
import org.t2framework.lucy.config.meta.InjectConfig;
import org.t2framework.lucy.inject.AbstractInjector.InjectInfo;

/**
 * 
 * {@.en {@link InjectorUtil} is an utility class for injection.}
 * 
 * <br />
 * 
 * {@.ja {@link InjectorUtil}はインジェクションするときのユーティリティクラスです.}
 * 
 * @author shot
 * 
 */
public class InjectorUtil {

	/**
	 * 
	 * {@.en }
	 * 
	 * <br />
	 * 
	 * {@.ja &#064;Injectが{@code ConfigContainer}内に存在するかどうかチェックします.}
	 * 
	 * @param container
	 * @return
	 */
	public static boolean hasInjectAnnotation(ConfigContainer container) {
		if (container == null) {
			return false;
		}
		return container.hasConfig(Inject.class);
	}

	/**
	 * 
	 * {@.en }
	 * 
	 * <br />
	 * 
	 * {@.ja {@code InjectConfig}が{@code ConfigContainer}内に存在するかチェックします.}
	 * 
	 * @param container
	 * @return
	 */
	public static boolean hasInjectConfiguration(ConfigContainer container) {
		if (container == null) {
			return false;
		}
		return container.hasConfig(InjectConfig.class);
	}

	public static boolean isInjectAnnotation(Config config) {
		if (config == null || config.hasAnnotation() == false) {
			return false;
		}
		return config.getAnnotation().annotationType() == Inject.class;
	}

	/**
	 * Get {@link Class}[] from {@link Inject}.types() or {@link InjectConfig}
	 * .getParameterTypes().
	 * 
	 * @param container
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public static Class[] getTypes(ConfigContainer container) {
		if (hasInjectAnnotation(container) == false
				&& hasInjectConfiguration(container) == false) {
			return null;
		}
		Class[] types = null;
		for (Config config : container) {
			Class<?> type = config.getType();
			if (type.equals(Inject.class)) {
				types = ((Inject) config.getAnnotation()).types();
				if (types.length == 1 && types[0] == Object.class) {
					types = null;
				}
				return types;
			} else if (type.equals(InjectConfig.class)) {
				return ((InjectConfig) config).getParameterTypes();
			}
		}
		return types;
	}

	/**
	 * 
	 * {@.en Get inject-able component from {@code LucyHandler}.}
	 * 
	 * <br />
	 * 
	 * {@.ja }
	 * 
	 * @param <T>
	 * @param type
	 * @param info
	 * @param namesIndex
	 * @param handler
	 * @return
	 */
	public static <T> T getInjectableComponent(Class<? super T> type,
			InjectInfo info, int namesIndex, InjectHandler<T> handler) {
		T value = null;
		if (isSimplyByType(info)) {
			value = handler.getFromType(type);
		} else {
			int namesLength = info.getNames().length;
			if (namesIndex < namesLength) {
				value = handler.getFromIdentifier(info.getNames()[namesIndex]);
			} else {
				value = handler.getFromType(type);
			}
		}
		return value;
	}

	private static boolean isSimplyByType(InjectInfo info) {
		return info == null || info.hasNames() == false;
	}

	/**
	 * 
	 * {@.en An interface for somehow looking component from Lucy.}
	 * 
	 * <br />
	 * 
	 * {@.ja }
	 * 
	 * @author shot
	 * 
	 * @param <T>
	 */
	public static interface InjectHandler<T> {
		@SuppressWarnings("unchecked")
		T getFromType(Class type);

		T getFromIdentifier(Object identifier);
	}
}
