package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.commons.util.Assertion;
import org.t2framework.lucy.Lifecycle;

/**
 * Convenient abstract Behavior class.
 * 
 * @author shot
 */
public abstract class AbstractBehavior implements Behavior {

	protected BeanDesc<Object> beanDesc;

	protected ClassDesc<Object> classDesc;

	protected MethodDesc methodDesc;

	protected PropertyDesc<Object> propertyDesc;

	protected Config config;

	/**
	 * Constructors are called by reflection.
	 * 
	 * @param beanDesc
	 * @param classDesc
	 * @param cd
	 */
	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, ClassDesc classDesc, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.classDesc = Assertion.notNull(classDesc);
		this.config = cd;
	}

	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, MethodDesc md, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.methodDesc = Assertion.notNull(md);
		this.config = Assertion.notNull(cd);
	}

	@SuppressWarnings("unchecked")
	public AbstractBehavior(BeanDesc beanDesc, PropertyDesc pd, Config cd) {
		this.beanDesc = Assertion.notNull(beanDesc);
		this.propertyDesc = Assertion.notNull(pd);
		this.config = Assertion.notNull(cd);
	}

	@Override
	public <T> void init() {
	}

	@Override
	public <T> void destroy() {
		beanDesc = null;
		classDesc = null;
		methodDesc = null;
		propertyDesc = null;
	}

	protected final BeanDesc<Object> getBeanDesc() {
		return beanDesc;
	}

	protected final ClassDesc<Object> getClassDesc() {
		return classDesc;
	}

	protected final MethodDesc getMethodDesc() {
		return methodDesc;
	}

	protected final PropertyDesc<Object> getPropertyDesc() {
		return propertyDesc;
	}

	protected final Config getConfig() {
		return config;
	}

	@Override
	public boolean isApplied(Lifecycle current) {
		return current != null && current == getLifecycle();
	}

}
