package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.commons.meta.ClassDesc;
import org.t2framework.commons.meta.Config;
import org.t2framework.commons.meta.MethodDesc;
import org.t2framework.commons.meta.PropertyDesc;
import org.t2framework.lucy.Lifecycle;
import org.t2framework.lucy.Lucy;

/**
 * Behavior is base interface deciding how Lucy behaves.The behavior class
 * always go with an configuration point just like InjectBehavior is for
 * &#064;Inject or InjectConfig.
 * 
 * <pre>
 * Behavior's constructor must have at least one of these:
 *  - {@link BeanDesc}, {@link ClassDesc}, {@link Config}
 *  - {@link BeanDesc}, {@link MethodDesc}, {@link Config}
 *  - {@link BeanDesc}, {@link PropertyDesc}, {@link Config}
 * </pre>
 * 
 * @author shot
 */
public interface Behavior {

	/**
	 * Initialize this {@link Behavior}.
	 * 
	 * @param <T>
	 */
	<T> void init();

	/**
	 * Execute {@link Behavior} with the instance representing T, and
	 * {@link Lucy}.
	 * 
	 * @param <T>
	 * @param t
	 * @param lucy
	 * @return
	 */
	<T> T execute(T t, Lucy lucy);

	/**
	 * Destroy this {@link Behavior}.
	 * 
	 * @param <T>
	 */
	<T> void destroy();

	/**
	 * Get {@link Lifecycle} that tells when to invoke this {@link Behavior}.
	 * 
	 * @return
	 */
	Lifecycle getLifecycle();

	/**
	 * True if this {@link Behavior} should be invoked with the given
	 * {@link Lifecycle}.
	 * 
	 * @param current
	 * @return
	 */
	boolean isApplied(Lifecycle current);
}
