package org.t2framework.lucy.spi;

import org.t2framework.commons.meta.BeanDesc;
import org.t2framework.lucy.BeanDescStrategy;
import org.t2framework.lucy.ConfigBinder;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyComponentRegistrar;

/**
 * Configuration class for {@link Lucy}.
 * 
 * @author shot
 */
public interface LucyConfiguration {

	/**
	 * Load {@link ConfigBinder}, the class for mapping annotation/xml
	 * configuration and {@link Behavior}.
	 * 
	 * @return
	 */
	ConfigBinder loadConfigBinder();

	/**
	 * Create {@link ComponentBuilder}.
	 * 
	 * @param beanDesc
	 * @param lucy
	 * @return
	 */
	ComponentBuilder createBuilder(BeanDesc<?> beanDesc, Lucy lucy);

	/**
	 * Get {@link BeanDescStrategy} as configurable strategy class for
	 * {@link BeanDesc}.
	 * 
	 * @param lucy
	 * @return
	 */
	BeanDescStrategy getBeanDescStrategy(Lucy lucy);

	/**
	 * Get {@link LucyComponentRegistrar} from the given resource as a key.
	 * 
	 * @param <R>
	 * @param resource
	 * @return
	 */
	<R> LucyComponentRegistrar<R> getAcceptableComponentRegistrar(R resource);
}
