package org.t2framework.lucy.ut;

import org.t2framework.commons.ut.DefaultUtPlugin;
import org.t2framework.lucy.Lucy;
import org.t2framework.lucy.LucyBootstrap;
import org.t2framework.lucy.impl.DefaultLucyConfiguration;
import org.t2framework.lucy.spi.LucyConfiguration;

/**
 * Test case plugin for {@link LucyTestCase}.
 * 
 * @author shot
 * 
 */
public class LucyUtPlugin extends DefaultUtPlugin {

	protected LucyConfiguration configuration;

	protected Object componentDefinition;

	public LucyUtPlugin(LucyTestCase testCase) {
		this(testCase, testCase.getLucyConfiguration());
	}

	public LucyUtPlugin(LucyTestCase testCase, LucyConfiguration configuration) {
		this(testCase, configuration, null);
	}

	public LucyUtPlugin(LucyTestCase testCase, LucyConfiguration configuration,
			Object componentDefinition) {
		super(testCase);
		this.configuration = configuration;
		this.componentDefinition = componentDefinition;
	}

	/**
	 * Begin test processing.
	 */
	@Override
	public void begin() throws Throwable {
		final LucyConfiguration configuration = getConfiguration();
		if (configuration == null) {
			throw new NullPointerException(
					"lucy configuration must not be null.");
		}
		LucyBootstrap.init(configuration, getComponentDefinition());
	}

	/**
	 * End test processing.
	 */
	@Override
	public void end() throws Throwable {
		LucyBootstrap.destroy();
	}

	/**
	 * Get {@link Lucy} for unit testing.
	 * 
	 * @return
	 */
	public Lucy getLucy() {
		return LucyBootstrap.getLucy();
	}

	protected LucyConfiguration getConfiguration() {
		return (configuration != null) ? configuration
				: new DefaultLucyConfiguration();
	}

	protected Object getComponentDefinition() {
		return componentDefinition;
	}
}
